'use strict';
















var _chalk = require('chalk');var _chalk2 = _interopRequireDefault(_chalk);
var _BaseReporter = require('./BaseReporter');var _BaseReporter2 = _interopRequireDefault(_BaseReporter);
var _utils = require('./utils');
var _getResultHeader = require('./getResultHeader');var _getResultHeader2 = _interopRequireDefault(_getResultHeader);function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { default: obj };} /**
                                                                                                                                                                                                                    * Copyright (c) 2014-present, Facebook, Inc. All rights reserved.
                                                                                                                                                                                                                    *
                                                                                                                                                                                                                    * This source code is licensed under the BSD-style license found in the
                                                                                                                                                                                                                    * LICENSE file in the root directory of this source tree. An additional grant
                                                                                                                                                                                                                    * of patent rights can be found in the PATENTS file in the same directory.
                                                                                                                                                                                                                    *
                                                                                                                                                                                                                    * 
                                                                                                                                                                                                                    */const ARROW = ' \u203A ';const FAIL_COLOR = _chalk2.default.bold.red;const SNAPSHOT_ADDED = _chalk2.default.bold.green;const SNAPSHOT_NOTE = _chalk2.default.dim;const SNAPSHOT_REMOVED = _chalk2.default.bold.red;const SNAPSHOT_SUMMARY = _chalk2.default.bold;const SNAPSHOT_UPDATED = _chalk2.default.bold.green;
const TEST_SUMMARY_THRESHOLD = 20;

const NPM_EVENTS = new Set([
'prepublish',
'publish',
'postpublish',
'preinstall',
'install',
'postinstall',
'preuninstall',
'uninstall',
'postuninstall',
'preversion',
'version',
'postversion',
'pretest',
'test',
'posttest',
'prestop',
'stop',
'poststop',
'prestart',
'start',
'poststart',
'prerestart',
'restart',
'postrestart']);


class SummaryReporter extends _BaseReporter2.default {




  constructor(globalConfig, options) {
    super();
    this._globalConfig = globalConfig;
    this._estimatedTime = 0;
    this._options = options;
  }

  // If we write more than one character at a time it is possible that
  // Node.js exits in the middle of printing the result. This was first observed
  // in Node.js 0.10 and still persists in Node.js 6.7+.
  // Let's print the test failure summary character by character which is safer
  // when hundreds of tests are failing.
  _write(string) {
    for (let i = 0; i < string.length; i++) {
      process.stderr.write(string.charAt(i));
    }
  }

  onRunStart(
  aggregatedResults,
  options)
  {
    super.onRunStart(aggregatedResults, options);
    this._estimatedTime = options.estimatedTime;
  }

  onRunComplete(contexts, aggregatedResults) {const
    numTotalTestSuites = aggregatedResults.numTotalTestSuites,testResults = aggregatedResults.testResults,wasInterrupted = aggregatedResults.wasInterrupted;
    if (numTotalTestSuites) {
      const lastResult = testResults[testResults.length - 1];
      // Print a newline if the last test did not fail to line up newlines
      // similar to when an error would have been thrown in the test.
      if (
      !this._globalConfig.verbose &&
      lastResult &&
      !lastResult.numFailingTests &&
      !lastResult.testExecError)
      {
        this.log('');
      }

      this._printSummary(aggregatedResults, this._globalConfig);
      this._printSnapshotSummary(
      aggregatedResults.snapshot,
      this._globalConfig);


      if (numTotalTestSuites) {
        const testSummary = wasInterrupted ?
        _chalk2.default.bold.red('Test run was interrupted.') :
        this._getTestSummary(
        contexts,
        this._options.pattern,
        this._options.testNamePattern,
        this._options.testPathPattern);

        this.log(
        (0, _utils.getSummary)(aggregatedResults, {
          estimatedTime: this._estimatedTime }) +

        '\n' +
        testSummary);

      }
    }
  }

  _printSnapshotSummary(
  snapshots,
  globalConfig)
  {
    if (
    snapshots.added ||
    snapshots.filesRemoved ||
    snapshots.unchecked ||
    snapshots.unmatched ||
    snapshots.updated)
    {
      let updateCommand;
      const event = process.env.npm_lifecycle_event;
      const prefix = NPM_EVENTS.has(event) ? '' : 'run ';
      const client = typeof process.env.npm_config_user_agent === 'string' &&
      process.env.npm_config_user_agent.match('yarn') !== null ?
      'yarn' :
      'npm';
      if (globalConfig.watch) {
        updateCommand = 'press `u`';
      } else if (event) {
        updateCommand = `run with \`${client + ' ' + prefix + event} -- -u\``;
      } else {
        updateCommand = 're-run with `-u`';
      }

      this.log(SNAPSHOT_SUMMARY('Snapshot Summary'));
      if (snapshots.added) {
        this.log(
        SNAPSHOT_ADDED(ARROW + (0, _utils.pluralize)('snapshot', snapshots.added)) +
        ` written in ${(0, _utils.pluralize)('test suite', snapshots.filesAdded)}.`);

      }

      if (snapshots.unmatched) {
        this.log(
        FAIL_COLOR(ARROW + (0, _utils.pluralize)('snapshot test', snapshots.unmatched)) +
        ` failed in ` +
        `${(0, _utils.pluralize)('test suite', snapshots.filesUnmatched)}. ` +
        SNAPSHOT_NOTE(
        'Inspect your code changes or ' +
        updateCommand +
        ' to update them.'));


      }

      if (snapshots.updated) {
        this.log(
        SNAPSHOT_UPDATED(ARROW + (0, _utils.pluralize)('snapshot', snapshots.updated)) +
        ` updated in ${(0, _utils.pluralize)('test suite', snapshots.filesUpdated)}.`);

      }

      if (snapshots.filesRemoved) {
        this.log(
        SNAPSHOT_REMOVED(
        ARROW + (0, _utils.pluralize)('obsolete snapshot file', snapshots.filesRemoved)) + (

        snapshots.didUpdate ?
        ' removed.' :
        ' found, ' +
        updateCommand +
        ' to remove ' + (
        snapshots.filesRemoved === 1 ? 'it' : 'them.') +
        '.'));

      }

      if (snapshots.unchecked) {
        this.log(
        FAIL_COLOR(
        ARROW + (0, _utils.pluralize)('obsolete snapshot', snapshots.unchecked)) + (

        snapshots.didUpdate ?
        ' removed.' :
        ' found, ' +
        updateCommand +
        ' to remove ' + (
        snapshots.filesRemoved === 1 ? 'it' : 'them') +
        '.'));

      }

      this.log(''); // print empty line
    }
  }

  _printSummary(
  aggregatedResults,
  globalConfig)
  {
    // If there were any failing tests and there was a large number of tests
    // executed, re-print the failing results at the end of execution output.
    const failedTests = aggregatedResults.numFailedTests;
    const runtimeErrors = aggregatedResults.numRuntimeErrorTestSuites;
    if (
    failedTests + runtimeErrors > 0 &&
    aggregatedResults.numTotalTestSuites > TEST_SUMMARY_THRESHOLD)
    {
      this.log(_chalk2.default.bold('Summary of all failing tests'));
      aggregatedResults.testResults.forEach(testResult => {const
        failureMessage = testResult.failureMessage;
        if (failureMessage) {
          this._write(
          (0, _getResultHeader2.default)(testResult, globalConfig) +
          '\n' +
          failureMessage +
          '\n');

        }
      });
      this.log(''); // print empty line
    }
  }

  _getTestSummary(
  contexts,
  testSelectionConfig,
  testNamePattern,
  testPathPattern)
  {
    const testInfo = testSelectionConfig.onlyChanged ?
    _chalk2.default.dim(' related to changed files') :
    testSelectionConfig.input !== '' ?
    _chalk2.default.dim(' matching ') + testPathPattern :
    '';

    const nameInfo = testNamePattern ?
    _chalk2.default.dim(' with tests matching ') + `"${testNamePattern}"` :
    '';

    const contextInfo = contexts.size > 1 ?
    _chalk2.default.dim(' in ') + contexts.size + _chalk2.default.dim(' projects') :
    '';

    return (
      _chalk2.default.dim('Ran all test suites') +
      testInfo +
      nameInfo +
      contextInfo +
      _chalk2.default.dim('.'));

  }}


module.exports = SummaryReporter;