'use strict';
















var _path = require('path');var _path2 = _interopRequireDefault(_path);
var _jestUtil = require('jest-util');
var _chalk = require('chalk');var _chalk2 = _interopRequireDefault(_chalk);
var _gracefulFs = require('graceful-fs');var _gracefulFs2 = _interopRequireDefault(_gracefulFs);
var _getMaxWorkers = require('./lib/getMaxWorkers');var _getMaxWorkers2 = _interopRequireDefault(_getMaxWorkers);
var _getTestPathPattern = require('./lib/getTestPathPattern');var _getTestPathPattern2 = _interopRequireDefault(_getTestPathPattern);
var _SearchSource = require('./SearchSource');var _SearchSource2 = _interopRequireDefault(_SearchSource);
var _updateArgv = require('./lib/updateArgv');var _updateArgv2 = _interopRequireDefault(_updateArgv);
var _TestRunner = require('./TestRunner');var _TestRunner2 = _interopRequireDefault(_TestRunner);
var _TestSequencer = require('./TestSequencer');var _TestSequencer2 = _interopRequireDefault(_TestSequencer);
var _testResultHelpers = require('./testResultHelpers');function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { default: obj };}function _asyncToGenerator(fn) {return function () {var gen = fn.apply(this, arguments);return new Promise(function (resolve, reject) {function step(key, arg) {try {var info = gen[key](arg);var value = info.value;} catch (error) {reject(error);return;}if (info.done) {resolve(value);} else {return Promise.resolve(value).then(function (value) {step("next", value);}, function (err) {step("throw", err);});}}return step("next");});};} /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        * Copyright (c) 2014-present, Facebook, Inc. All rights reserved.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        * This source code is licensed under the BSD-style license found in the
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        * LICENSE file in the root directory of this source tree. An additional grant
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        * of patent rights can be found in the PATENTS file in the same directory.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        * 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        */const setConfig = (contexts, newConfig) => contexts.forEach(context => context.config = Object.freeze(Object.assign({}, context.config, newConfig)));

const formatTestPathPattern = pattern => {
  const testPattern = pattern.testPathPattern;
  const input = pattern.input;
  const formattedPattern = `/${testPattern || ''}/`;
  const formattedInput = pattern.shouldTreatInputAsPattern ?
  `/${input || ''}/` :
  `"${input || ''}"`;
  return input === testPattern ? formattedInput : formattedPattern;
};

const getNoTestsFoundMessage = (testRunData, pattern) => {
  if (pattern.onlyChanged) {
    return (
      _chalk2.default.bold(
      'No tests found related to files changed since last commit.\n') +

      _chalk2.default.dim(
      pattern.watch ?
      'Press `a` to run all tests, or run Jest with `--watchAll`.' :
      'Run Jest without `-o` to run all tests.'));


  }

  const pluralize = (word, count, ending) =>
  `${count} ${word}${count === 1 ? '' : ending}`;
  const testPathPattern = formatTestPathPattern(pattern);
  const individualResults = testRunData.map(testRun => {
    const stats = testRun.matches.stats || {};
    const config = testRun.context.config;
    const statsMessage = Object.keys(stats).
    map(key => {
      if (key === 'roots' && config.roots.length === 1) {
        return null;
      }
      const value = config[key];
      if (value) {
        const matches = pluralize('match', stats[key], 'es');
        return `  ${key}: ${_chalk2.default.yellow(value)} - ${matches}`;
      }
      return null;
    }).
    filter(line => line).
    join('\n');

    return testRun.matches.total ?
    `In ${_chalk2.default.bold(config.rootDir)}\n` +
    `  ${pluralize('file', testRun.matches.total || 0, 's')} checked.\n` +
    statsMessage :
    `No files found in ${config.rootDir}.\n` +
    `Make sure Jest's configuration does not exclude this directory.` +
    `\nTo set up Jest, make sure a package.json file exists.\n` +
    `Jest Documentation: ` +
    `facebook.github.io/jest/docs/configuration.html`;
  });
  return (
    _chalk2.default.bold('No tests found') +
    '\n' +
    individualResults.join('\n') +
    '\n' +
    `Pattern: ${_chalk2.default.yellow(testPathPattern)} - 0 matches`);

};

const getTestPaths = (() => {var _ref = _asyncToGenerator(function* (
  globalConfig,
  context,
  pattern,
  argv,
  outputStream)
  {
    const source = new _SearchSource2.default(context);
    let data = yield source.getTestPaths(pattern);
    if (!data.tests.length) {
      if (pattern.onlyChanged && data.noSCM) {
        if (globalConfig.watch) {
          // Run all the tests
          (0, _updateArgv2.default)(argv, 'watchAll', { noSCM: true });
          pattern = (0, _getTestPathPattern2.default)(argv);
          data = yield source.getTestPaths(pattern);
        } else {
          new _jestUtil.Console(outputStream, outputStream).log(
          'Jest can only find uncommitted changed files in a git or hg ' +
          'repository. If you make your project a git or hg ' +
          'repository (`git init` or `hg init`), Jest will be able ' +
          'to only run tests related to files changed since the last ' +
          'commit.');

        }
      }
    }
    return data;
  });return function getTestPaths(_x, _x2, _x3, _x4, _x5) {return _ref.apply(this, arguments);};})();

const processResults = (runResults, options) => {
  if (options.testResultsProcessor) {
    /* $FlowFixMe */
    runResults = require(options.testResultsProcessor)(runResults);
  }
  if (options.isJSON) {
    if (options.outputFile) {
      const outputFile = _path2.default.resolve(process.cwd(), options.outputFile);

      _gracefulFs2.default.writeFileSync(
      outputFile,
      JSON.stringify((0, _jestUtil.formatTestResults)(runResults)));

      process.stdout.write(
      `Test results written to: ` +
      `${_path2.default.relative(process.cwd(), outputFile)}\n`);

    } else {
      process.stdout.write(JSON.stringify((0, _jestUtil.formatTestResults)(runResults)));
    }
  }
  return options.onComplete && options.onComplete(runResults);
};

const runJest = (() => {var _ref2 = _asyncToGenerator(function* (
  globalConfig,
  contexts,
  argv,
  outputStream,
  testWatcher,
  startRun,
  onComplete,





  printNoTestsMessage)




  {
    const maxWorkers = (0, _getMaxWorkers2.default)(argv);
    const testSelectionConfig = (0, _getTestPathPattern2.default)(argv);
    const sequencer = new _TestSequencer2.default();
    let allTests = [];
    const testRunData = yield Promise.all(
    contexts.map((() => {var _ref3 = _asyncToGenerator(function* (context) {
        const matches = yield getTestPaths(
        globalConfig,
        context,
        testSelectionConfig,
        argv,
        outputStream);

        allTests = allTests.concat(matches.tests);
        return { context, matches };
      });return function (_x14) {return _ref3.apply(this, arguments);};})()));


    allTests = sequencer.sort(allTests);

    if (argv.listTests) {
      console.log(JSON.stringify(allTests.map(function (test) {return test.path;})));
      onComplete && onComplete((0, _testResultHelpers.makeEmptyAggregatedTestResult)());
      return null;
    }

    if (!allTests.length) {
      if (printNoTestsMessage) {
        printNoTestsMessage(outputStream, testRunData, testSelectionConfig);
      } else {
        new _jestUtil.Console(outputStream, outputStream).log(
        getNoTestsFoundMessage(testRunData, testSelectionConfig));

      }
    } else if (
    allTests.length === 1 &&
    globalConfig.silent !== true &&
    globalConfig.verbose !== false)
    {
      // $FlowFixMe
      globalConfig = Object.freeze(
      Object.assign({}, globalConfig, { verbose: true }));

    }

    // When using more than one context, make all printed paths relative to the
    // current cwd. rootDir is only used as a token during normalization and
    // has no special meaning afterwards except for printing information to the
    // CLI.
    setConfig(contexts, { rootDir: process.cwd() });

    const results = yield new _TestRunner2.default(globalConfig, {
      maxWorkers,
      pattern: testSelectionConfig,
      startRun,
      testNamePattern: argv.testNamePattern,
      testPathPattern: formatTestPathPattern(testSelectionConfig) }).
    runTests(allTests, testWatcher);

    sequencer.cacheResults(allTests, results);

    return processResults(results, {
      isJSON: argv.json,
      onComplete,
      outputFile: argv.outputFile,
      testResultsProcessor: globalConfig.testResultsProcessor });

  });return function runJest(_x6, _x7, _x8, _x9, _x10, _x11, _x12, _x13) {return _ref2.apply(this, arguments);};})();

module.exports = runJest;